/* **************************************************************************																					
 *  Software License Agreement
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  This is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License (version 2) as published by 
 *  the Free Software Foundation AND MODIFIED BY OpenPicus team.
 *  
 *  ***NOTE*** The exception to the GPL is included to allow you to distribute
 *  a combined work that includes OpenPicus code without being obliged to 
 *  provide the source code for proprietary components outside of the OpenPicus
 *  code. 
 *  OpenPicus software is distributed in the hope that it will be useful, but 
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 *  more details. 
 * 
 * 
 * Warranty
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * WE ARE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 **************************************************************************/
 
 
/****************************************************************************
  SECTION 	Include
****************************************************************************/

#include "TCPIP Stack/TCPIP.h"
#if defined(STACK_USE_HTTP2_SERVER)
#define __HTTPAPP_C

const char *StateLabel[]={"Sending Parameters","Wait Sun/Grid","Checking Grid","Measuring Riso",
"DcDc Start","Inverter","Run ","Recovery","Pause","Ground Fault",
"OTH Fault","Address Setting","Self Test","Self Test Fail","Sensor Test + Meas.Riso",
"Leak Fault","Waiting for manual reset",
"Internal Error E026","Internal Error E027","Internal Error E028","Internal Error E029","Internal Error E030",
"Sending Wind Table","Failed Sending table","UTH Fault","Remote OFF","Interlock Fail",
"Executing Autotest","Waiting Sun","Temperature Fault","Fan Staucked",
"Int. Com. Fault","Slave Insertion","DC Switch Open","TRAS Switch Open","MASTER Exclusion",
"Auto Exclusion","Erasing Internal EEprom","Erasing External EEpro","Counting EEprom","Freeze"};


const char *AlarmLabel[]={"No Alarm","Sun Low","Input OC","Input UV","Input OV","Sun Low","No Parameters","Bulk OV","Comm.Error","Output OC","IGBT Sat","Bulk UV",
"Internal error","Grid Fail","Bulk Low","Ramp Fail","Dc/Dc Fail","Wrong Mode","Ground Fault","Over Temp.","Bulk Cap Fail",
"Inverter Fail","Start Timeout","Ground Fault","Degauss error","Ileak sens.fail","DcDc Fail","Self Test Error 1",
"Self Test Error 2","Self Test Error 3","Self Test Error 4","DC inj error","Grid OV","Grid UV","Grid OF",
"Grid UF","Z grid Hi","Internal error","Riso Low","Vref Error","Error Meas V","Error Meas F","Error Meas Z",
"Error Meas Ileak","Error Read V","Error Read I","Table fail","Fan Fail","UTH","Interlock fail","Remote Off",
"Vout Avg errror","Battery low","Clk fail","Input UC","Zero Power","Fan Stucked","DC Switch Open",
"Tras Switch Open","AC Switch Open","Bulk UV","Autoexclusion","Grid df/dt","Den switch Open","box fail"};

typedef struct {
  int           Indirizzo;
  int           EnergiaAttiva;
  float         ProduzioneTotale;
  float         ProduzioneGiornaliera;
  float         TensioneCampo;
  float         CorrenteCampo;
  float         TensioneRete;
  float         CorrenteRete;
  float         FrequenzaRete;
  int           Stato;
  int           Allarme;
  float         TempCPU;
  float         TempIGBT;
  int           ReleAutoconsumo;
  char          UpdateDataTime[17]; 
} _inverter;

extern _inverter  inverterdata;

/****************************************************************************
  FUNCTION	HTTP_IO_RESULT HTTPExecuteGet(void)
	
  This function processes every GET request from the pages. For futher 
  information, please see the "standard webserver" example provided.
*****************************************************************************/
HTTP_IO_RESULT HTTPExecuteGet(void)
{
	//BYTE *ptr;
	BYTE filename[20];
	
	// STEP #1:
	// The function MPFSGetFilename retrieves the name of the requested cgi,
	// in this case "leds.cgi" and puts it inside the filename variable.
	// Make sure BYTE filename[] above is large enough for your longest name
	MPFSGetFilename(curHTTP.file, filename, 20);

	// STEP #2:
	// Handling of the cgi requests, in this case we have only "leds.cgi" but
	// it would be possible to have any other cgi request, depending on the webpage
	
	
	return HTTP_IO_DONE;
}


/****************************************************************************
  Section:
	POST Form Handlers
  ***************************************************************************/
#if defined(HTTP_USE_POST)

/*****************************************************************************
  Function:
	HTTP_IO_RESULT HTTPExecutePost(void)

  	This function processes every POST request from the pages. 
  ***************************************************************************/
HTTP_IO_RESULT HTTPExecutePost(void)
{
	// Resolve which function to use and pass along
	BYTE filename[20];

	// Load the file name
	// Make sure BYTE filename[] above is large enough for your longest name
	MPFSGetFilename(curHTTP.file, filename, sizeof(filename));

	return HTTP_IO_DONE;
}

#endif 



/***************************************************************************
  SECTION	Dynamic Variable Callback Functions
  
  In this section are managed the "dynamic variables" of the webserver.
  Dynamic variables are contained in the status.xml file requested by the 
  webpage. 
  For each dynamic variable a callback function named HTTPPrint_varname 
  must be created.
****************************************************************************/

void HTTPPrint_inverter(WORD num){

  char value[50];

  switch(num) {
    case 0:
      if(inverterdata.Indirizzo==0){
        sprintf(value,"Inverter non trovato");
      } else {
        sprintf(value,"0x%02d",inverterdata.Indirizzo);
      }
    break;
    case 1:
      sprintf(value,"%d",inverterdata.EnergiaAttiva);
    break;
    case 2:
      sprintf(value,"%.2f",(double)inverterdata.ProduzioneTotale);
    break;
    case 3:
      sprintf(value,"%.2f",(double)inverterdata.ProduzioneGiornaliera);
    break;
    case 4:
      sprintf(value,"%.2f",(double)inverterdata.TensioneCampo);
    break;
    case 5:
      sprintf(value,"%.2f",(double)inverterdata.CorrenteCampo);
    break;
    case 6:
      sprintf(value,"%.2f",(double)inverterdata.TensioneRete);
    break;
    case 7:
      sprintf(value,"%.2f",(double)inverterdata.CorrenteRete);
    break;
    case 8:
      sprintf(value,"%.2f",(double)inverterdata.FrequenzaRete);
    break;
    case 9:
	  TCPPutString(sktHTTP, (BYTE*)"<img src=images/");
      if(inverterdata.Indirizzo>0){
        if(inverterdata.Stato==6)
          sprintf(value,"green.png></td><td class=listbg><b>%s",StateLabel[inverterdata.Stato]);
        else
          sprintf(value,"yellow.png></td><td class=listbg><b>%s",StateLabel[inverterdata.Stato]);
      } else 
        sprintf(value,"grey.png></td><td class=listbg><b>OFF LINE");
    break;
    case 10:
       TCPPutString(sktHTTP, (BYTE*)"<img src=images/");
      if(inverterdata.Indirizzo>0){
        if(inverterdata.Allarme>0)
          sprintf(value,"red.png></td><td class=listbg><b>%s",AlarmLabel[inverterdata.Allarme]);
        else
          sprintf(value,"grey.png></td><td class=listbg><b>%s",AlarmLabel[inverterdata.Allarme]);
      } else 
        sprintf(value,"grey.png></td><td class=listbg><b>OFF LINE");
    break;
    case 11:
      sprintf(value,"%.2f",(double)inverterdata.TempCPU);
    break;
    case 12:
      sprintf(value,"%.2f",(double)inverterdata.TempIGBT);
    break;
    case 13:
       TCPPutString(sktHTTP, (BYTE*)"<img src=images/");
      if(inverterdata.ReleAutoconsumo)
        sprintf(value,"green.png></td><td class=listbg><b>ON");
      else
        sprintf(value,"grey.png></td><td class=listbg><b>OFF");
    break;
	case 14:
	  sprintf(value,"%s",inverterdata.UpdateDataTime);
	break;
  }

  TCPPutString(sktHTTP, (BYTE*)value);
}


/****************************************************************************
  SECTION 	Authorization Handlers
****************************************************************************/
 

/*****************************************************************************
  FUNCTION	BYTE HTTPNeedsAuth(BYTE* cFile)

  This function is used by the stack to decide if a page is access protected.
  If the function returns 0x00, the page is protected, if returns 0x80, no 
  authentication is required
*****************************************************************************/
#if defined(HTTP_USE_AUTHENTICATION)
BYTE HTTPNeedsAuth(BYTE* cFile)
{
	//	If you want to restrict the access to some page, include it in the folder "protect"
	//	here you can change the folder, or add others
	if(memcmp(cFile, (void*)"protect", 7) == 0)
		return 0x00;		// Authentication will be needed later

	return 0x80;			// No authentication required
}
#endif

/*****************************************************************************
  FUNCTION	BYTE HTTPCheckAuth(BYTE* cUser, BYTE* cPass)
	
  This function checks if username and password inserted are acceptable

  ***************************************************************************/
#if defined(HTTP_USE_AUTHENTICATION)
BYTE HTTPCheckAuth(BYTE* cUser, BYTE* cPass)
{
	if(strcmp((char *)cUser,(char *)"admin") == 0
		&& strcmp((char *)cPass, (char *)"flyport") == 0)
		return 0x80;		// We accept this combination

	return 0x00;			// Provided user/pass is invalid
}
#endif

#endif
