/* **************************************************************************																					
 *                                OpenPicus                 www.openpicus.com
 *                                                            italian concept
 * 
 *            openSource wireless Platform for sensors and Internet of Things	
 * **************************************************************************
 *  FileName:        f_manager.c
 *  Dependencies:    Microchip configs files
 *  Module:          FlyPort generic module
 *  Compiler:        Microchip C30 v3.12 or higher
 *
 *
 *  Software License Agreement
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  This is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License (version 2) as published by 
 *  the Free Software Foundation AND MODIFIED BY OpenPicus team.
 *  
 *  ***NOTE*** The exception to the GPL is included to allow you to distribute
 *  a combined work that includes OpenPicus code without being obliged to 
 *  provide the source code for proprietary components outside of the OpenPicus
 *  code. 
 *  OpenPicus software is distributed in the hope that it will be useful, but 
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 *  more details. 
 * 
 * 
 * Warranty
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * WE ARE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 **************************************************************************/
#include "f_manager.h"
#include "taskFlyport.h"

FATFS Fatfs;

//	static function prototypes
static void sd_Hw_Init();
static BOOL disk_Unmount();
static BOOL disk_Mount();


/******************************************************************************
 *	void sd_Init()
 *
 *	Initializes the hardware/software SD card module. If the SD card is not 
 *	inserted, loops waiting to mount it until the SD ready is ready.
 *
 *	Return:
 *	-
 *****************************************************************************/	
/*
void sd_Init()
{
	//	----------	FILE SYSTEM INITIALIZATION	----------
	//	SD hardware initialization
	sd_Hw_Init();
			
	printdebug("Initializing SD card...\r\n");

	if (disk_Mount())
		printdebug("SD software module initialized!\r\n");
	else
		printdebug("SD software module NOT initialized!\r\n");
		
	// Check to detect SD card insertion
	while (disk_status(0))
	{
		disk_Unmount();
		vTaskDelay(50);
		printdebug(".");
		disk_Mount();
	}
	printdebug("SD card initialized and ready!\r\n");
	
}
*/
BYTE sd_Init() {
  DWORD      T0;
  //----------FILE SYSTEM INITIALIZATION	----------
  sd_Hw_Init(); //SD hardware initialization
  printdebug("Initializing SD card...\r\n");
  if (disk_Mount())
    printdebug("SD software module initialized!\r\n");
  else {
    printdebug("SD software module NOT initialized!\r\n");
    return 0;
  }
  // Check to detect SD card insertion
  T0 = TickGet();
  while (disk_status(0)) {
    disk_Unmount();
    vTaskDelay(50);
    printdebug(".");
    disk_Mount();
    if((TickGet()-T0)> 30*TICK_SECOND) return -1;
  }
  printdebug("SD card initialized and ready!\r\n");
  return 1; 
}

/******************************************************************************
 *	BOOL check_File(char* nfile_check)
 *
 *	Checks for a file existence
 *
 *	Return:
 *	TRUE - the file exists
 *	FALSE - The file doen't exist
 *****************************************************************************/
BOOL check_File(char* nfile_check)
{
	FRESULT file_res;
	FIL file_check;
	//	Open file; it must be specified the mode (now it is set to read only existing files)
	file_res = f_open(&file_check, nfile_check, FA_OPEN_EXISTING);
	
	//	Check error
	if (file_res != FR_OK)
	{
		f_close(&file_check);
		return FALSE;
	}
	else 
	{
		f_close(&file_check);
		return TRUE;
	}
}

BOOL create_File(char *nfile_create)
{
	FRESULT file_res;
	FIL file_create;
	file_res = f_open(&file_create, nfile_create, FA_CREATE_ALWAYS|FA_WRITE);
	
	if (file_res != FR_OK)
	{
		f_close(&file_create);
		return FALSE;
	}
	f_close(&file_create);
	return TRUE;
}


BOOL append_File(char *nfile_append, char *buff_to_app, unsigned int to_write)
{
	FRESULT file_res;
	FIL file_app;
	unsigned int len;
	
	//	File opening
	file_res = f_open(&file_app, nfile_append, FA_OPEN_EXISTING|FA_WRITE);
	if (file_res != FR_OK)
	{
		f_close(&file_app);
		return FALSE;
	}	
	
	//	Seek to the end of file
	file_res = f_lseek(&file_app, f_size(&file_app));
	if (file_res != FR_OK)
	{
		printdebug("ERRORE nel seek\n");
		f_close(&file_app);
		return FALSE;		
	}
	
	//	Append of the text at the end of file
	file_res = f_write(&file_app, buff_to_app, to_write, &len);
	if ( (file_res != FR_OK) || (len != to_write) )
	{
		printdebug("ERRORE nel write\n");
		f_close(&file_app);
		return FALSE;
	}
	f_close(&file_app);
	return TRUE;
}


BOOL write_File(char *file_name, char *buff_write, unsigned int len_write, BYTE mode) {
  FRESULT file_res;
  FIL file_app;
  unsigned int len;
  
  if(mode==0){
    file_res = f_open(&file_app, file_name, FA_OPEN_EXISTING|FA_WRITE);
    if (file_res != FR_OK) {
      mode=1;
    } else {
      //	Seek to the end of file
      file_res = f_lseek(&file_app, f_size(&file_app));
      if (file_res != FR_OK){
        printdebug("ERRORE nel seek\n");
        f_close(&file_app);
        return FALSE;		
      }
    }
  }

  if(mode==1){
    file_res = f_open(&file_app, file_name, FA_CREATE_ALWAYS|FA_WRITE);
    if (file_res != FR_OK) {
      f_close(&file_app);
      return FALSE;
    }
  }
  
  //	Append of the text at the end of file
  file_res = f_write(&file_app, buff_write, len_write, &len);
  if ( (file_res != FR_OK) || (len != len_write) ) {
    printdebug("ERRORE nel write\n");
    f_close(&file_app);
    return FALSE;
  }
  f_close(&file_app);
  return TRUE;
}



/******************************************************************************
 *	static void sd_Hw_Init()
 *
 *	Internal hardware SD card initialization, used by sd_Init(). Not to call
 *	in main application.
 *
 *	Return:
 *	-
 *****************************************************************************/	
static void sd_Hw_Init() {
  //Init GPIO SD su grouve
  IOInit(p7,	SPI_IN);
  IOInit(p18, SPICLKOUT);
  IOInit(p17, SPI_OUT);
  // init CS line as ouput, default high
  IOInit(p18, out); // SD-CS line
  IOPut (p18, on);
  // sd card detect line as input, with pullup
  IOInit(p19, inup); // SD-CS line
  // pullup and pulldown resistors:
  _CN24PDE = 1;	// SCK pulldown
  _CN26PUE = 1;	// SDO pullup
  _CN72PUE = 1;	// SDI pullup
  
  // FAT32 storage init
  #ifdef INTERRUPT
    // Start Timer4 in interval time of 1ms
    PR4 = 16000;
    T4CONbits.TON=1;
    _T4IE=1;
    T4CONbits.TCKPS0=0;
    T4CONbits.TCKPS1=0;
  #endif
  
  /*
	//SPI2 pins configuration:
	IOInit(sdsi, SPI_IN);
	IOInit(sdsc, SPICLKOUT);
	IOInit(sdso, SPI_OUT);
	// init CS line as ouput, default high
	IOInit(sdcs, out); // SD-CS line
	IOPut(sdcs, on);
	
	// sd card detect line as input, with pullup
	IOInit(sddetect, inup); // SD-CS line
	
	//Pullup and pulldown resitors:
	_CN53PDE = 1;  // CLK LOW
	_CN55PUE = 1;  // SDO HIGH
	_CN49PUE = 1;  // SDI HIGH
		
	#ifdef INTERRUPT
	// Start Timer4 in interval time of 1ms
		PR4 = 16000;
		T4CONbits.TON=1;
		_T4IE=1;
		T4CONbits.TCKPS0=0;
		T4CONbits.TCKPS1=0;
	#endif
	
	*/
}


/******************************************************************************
 *	static BOOL disk_Mount()
 *
 *	Mount the SD module. Internal static funtcion not to call from main 
 *	application.
 *
 *	Return:
 *	TRUE - the initialization succeded
 *	FALSE - the initialization doesn't succeded (SD card not inserted?)
 *****************************************************************************/	
static BOOL disk_Mount()
{	
	if ( f_mount(0, &Fatfs) == FR_OK)
	{
		disk_initialize(0);
		return TRUE;
	}
	else
		return FALSE;
}


/******************************************************************************
 *	static BOOL disk_Mount()
 *
 *	Unmount a previously mounted SD module. Internal static funtcion not to 
 *	call from main application.
 *
 *	Return:
 *	TRUE - the unmount succeded
 *	FALSE - the unmount doesn't succeded (SD card not mounted?)
 *****************************************************************************/	
static BOOL disk_Unmount()
{
	if ( f_mount(0, NULL) == FR_OK)
	{
		return TRUE;
	}
	else
		return FALSE;
}


/******************************************************************************
 *	static void close_File(FIL *file_close)
 *
 *	Closes a previously open file. Internal static function not to use in main 
 *	application.
 *****************************************************************************/
void close_File(FIL *file_close)
{
	f_close(file_close);
}




void printdebug(char* str)
{
#ifdef _DEBUG
	UARTWrite(1, str);
#endif
}
