/**********************************************************************
File Name:              taskFlyport.c
Dev:                    Daniele De Santis
Email:                  desantis@areasx.com
Web:                    http://www.areasx.com

Versione:               0.0
Data:                   24/09/12
Note Hardware:  
Note Software: 
**********************************************************************/
#include "taskFlyport.h"
#include "libpic30.h"
#include "daytime.h"
#include "diskio.h"
#include "f_manager.h"

#define _APP_NAME     "InverterConnectorWi-Fi"
#define _APP_VER      "1.0 beta"

#define sdcs          p9
#define sdso          p17
#define sdsc          p18
#define sdsi          p7
#define sddetect      p19

#define   DEBUG               //Abilita il debug su console
//#define   OPENPICUS_WIFI      //Configura il programma per operarte su FlyPort Wi-Fi. commentare su modulo ethernet

//Define Server DayTime
#define   NTP_SRV             "time.ien.it"
/***********************************************************************/


// SETUP


/***********************************************************************/

//Configurazione Parametri LAN
#define   LAN_IP            "192.168.0.100" //Indirizzo IP FlyPort
#define   LAN_MASK          "255.255.255.0" //Netmask
#define   LAN_DNS1          "195.110.128.1" //Indirizzo Primo server DNS
#define   LAN_DNS2          "212.48.4.11"   //Indirizzo Secondo server DNS
#define   LAN_GW            "192.168.0.1"   //Indirizzo ip del Gateway per l'accesso ad internet

//Configurazione parametri Wi-Fi solo su Ppenpicus Wi-Fi
#define   WI_FI_SSID        "www.areasx.com"                    //SSID access point 
#define   WI_FI_SECURITY    WF_SECURITY_WPA2_WITH_PASS_PHRASE   //Protezione WPA2
//#define   WI_FI_SECURITY    WF_SECURITY_WPA_WITH_PASS_PHRASE  //Protezione WAP
//#define   WI_FI_SECURITY    WF_SECURITY_WEP_104               //Protezione WEP 128
//#define   WI_FI_SECURITY    WF_SECURITY_OPEN                  //Nessuna Protezione WI_FI_PASS_PHRASE ""
#define   WI_FI_PASS_PHRASE  "xxxxxxxxx"                        //Password rete Wi-Fi

//Configurazione parametri WEB Server Remoto
#define   WEB_IP          "www.serverwebremoto.it" //Indirizzo del WEB server remoto
#define   WEB_PORT        "80"                     //Porta TCP-IP in ascolto del server WEB remoto
#define   WEB_URL         "/gateway_inverter.php"  //URL dello script gateway
#define   WEB_TIMEOUT     30                       //Timeout connessione in Secondi
#define   TXHTTP_PASSWORD "1234"                   //Configurazione password di trasmissione POST HTTP

//Configurazione parametri INVERTER su LAN
#define   INVERTER_IP         "192.168.0.101" //Indirizzo IP convertitore Sollae
#define   INVERTER_PORT       "1470"          //Porta TCP-IP in ascolto del Sollae
#define   INVERTER_TIMEOUT    30              //Timeout connessione in Secondi
#define   INVERTER_ADDRESS    0x00            //Address dell'inverter (0x00 ricerca automatica)

//Gestione Rele' AUTOCONSUMO
#define   SOGLIA_AUTOCONSUMO  1000  //Soglia in W attivazione rel autoconsumo
#define   TIMEOUT_AUTOCONSUMO 120   //Timeout in Sec reset rele' autocunsumo

/***********************************************************************/


/***********************************************************************/

//LED STATUS (VERDE)
#define   STATUS_LED_ON         IOPut(D5Out , 1)   //OUT4 Net su flyport
#define   STATUS_LED_OFF        IOPut(D5Out , 0)
#define   STATUS_LED_TOGGLE     IOPut(D5Out , IOGet(D5Out) ^ 0x01)


//LED (ROSSO)
#define   ERROR_LED_ON         IOPut(p4 , 1)
#define   ERROR_LED_OFF        IOPut(p4 , 0)
#define   ERROR_LED_TOGGLE     IOPut(p4 , IOGet(p4) ^ 0x01)

#define   ALERT_LED_ON         ERROR_LED_ON;  STATUS_LED_ON
#define   ALERT_LED_OFF        ERROR_LED_OFF; STATUS_LED_OFF

//RELE' 
#define   RELE_ON               IOPut(p6 , 1);   //Rele' su P6
#define   RELE_OFF              IOPut(p6, 0);
#define   RELE_TOGGLE           IOPut(p6, IOGet(p6) ^ 0x01)
#define   RELE_STATE            IOGet(p6)

#define   PORT_RS232            1   //Debug

#define   CRC_POLY        0x8408
//Serial number
#define CMD_SN(AD)    TxData[0]=AD; TxData[1]=0x3f; TxData[2]=0x00; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Produzione Totale
#define CMD_PT(AD)    TxData[0]=AD; TxData[1]=0x4e; TxData[2]=0x05; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Produzione Giornaliera
#define CMD_PG(AD)    TxData[0]=AD; TxData[1]=0x4e; TxData[2]=0x00; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Energia attiva
#define CMD_EA(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x03; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Tensione campo stringa 1
#define CMD_TC1(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x17; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Corrente campo stringa 1
#define CMD_CC1(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x19; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Tensione campo stringa 2
#define CMD_TC2(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x1A; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Corrente campo stringa 2
#define CMD_CC2(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x1B; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Tensione Rete
#define CMD_TR(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x01; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Corrente Rete
#define CMD_CR(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x02; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Frequenza Rete
#define CMD_FR(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x04; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Temperatura CPU
#define CMD_TC(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x15; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Temperatura IGBT
#define CMD_TI(AD)    TxData[0]=AD; TxData[1]=0x3b; TxData[2]=0x16; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;
//Stato e Allarme
#define CMD_ST(AD)    TxData[0]=AD; TxData[1]=0x32; TxData[2]=0x00; TxData[3]=0x00; TxData[4]=0x00; TxData[5]=0x00; TxData[6]=0x00; TxData[7]=0x00;

//Prototipi
int   CRC(char *cmd, unsigned char length);
int   SendCmd(char *cmd,unsigned char timeout);
int   OpenSocket(char* rIP, char* rPort, unsigned char timeout);
int   CloseSocket();
float ConvertFloat(char *buff);
long  ConvertLong(char *buff);
void  GetMD5(char *md5, char *buff);
int   HttpRequest(char *webip, char *webport, char *weburl, char *webquery,int webtimeout);
int   ParserXML(char *xml_buffer, char *start_tag, char *end_tag, int remove_tag, char *string_return);

//Struttura dati
typedef struct {
  int           Indirizzo;
  int           EnergiaAttiva;
  float         ProduzioneTotale;
  float         ProduzioneGiornaliera;
  float         TensioneCampo;
  float         CorrenteCampo;
  float         TensioneRete;
  float         CorrenteRete;
  float         FrequenzaRete;
  int           Stato;
  int           Allarme;
  float         TempCPU;
  float         TempIGBT;
  int           ReleAutoconsumo;
  char          UpdateDataTime[17];
} _inverter;


//Variabili globali
char                RxData[64];
char                TxData[64];
char                debug[100];
_inverter           inverterdata;
char                PostReturn[255];
TCP_SOCKET          TcpID = INVALID_SOCKET;

//Calcolo CRC protocollo AURORA
int CRC(char *cmd, unsigned char length) {
  unsigned char i;
  unsigned int tmp;
  unsigned int crc = 0xffff;
  if (length == 0) return (~crc);
  do {
    for (i=0, tmp=(unsigned int)0xff & *cmd++; i < 8; i++, tmp >>= 1){
      if ((crc & 0x0001) ^ (tmp & 0x0001)) crc = (crc >> 1) ^ CRC_POLY;
      else  crc >>= 1;
    }
  } while (--length);
  crc = ~crc;
  return crc;
}

int OpenSocket(char* rIP, char* rPort, unsigned char timeout){
  DWORD         T0;
  if(TcpID == INVALID_SOCKET)  TcpID = TCPClientOpen (rIP, rPort);
  T0 = TickGet();
  while(TcpID==INVALID_SOCKET){
    if(TickGet() - T0 > (timeout*TICK_SECOND)) { 
      return 0;
    }
    vTaskDelay(50);
  }
  T0 = TickGet();
  while(TCPisConn(TcpID)==FALSE){
    if((TickGet()-T0)> (timeout*TICK_SECOND)) {		
      return 0;
    }
    vTaskDelay(50);
  }
  return 1;
}

int CloseSocket(){
  DWORD         T0;
  TCPClientClose(TcpID);
  while(TCPisConn(TcpID)==TRUE){
    if((TickGet()-T0)> 30*TICK_SECOND) {
      return 0;
    }
    vTaskDelay(50);
  }
  TcpID = INVALID_SOCKET;
  return 1;
}

//Invia comando all'inverter AURORA
int SendCmd(char *cmd,unsigned char timeout){
  unsigned int mycrc;
  unsigned int rxsize=0;
  DWORD t0;
  mycrc = CRC(cmd, 8);
  TxData[8] = (mycrc & 0xFF);
  TxData[9] = ((mycrc>>8) & 0xFF);
  
  TCPWrite (TcpID, TxData, 10);
  t0=TickGet();
  do {
    rxsize = TCPRxLen(TcpID); 
    if(rxsize>0){
      t0=TickGet();
      TCPRead (TcpID, RxData, rxsize);
      break;
    }
    if((TickGet()-t0) > timeout*TICK_SECOND) {
      break;
    }
  } while(TCPisConn(TcpID));
  if(rxsize>=8){ //Verifica CRC
    mycrc = CRC(RxData, 6);
    if(((unsigned char)RxData[6]==(unsigned char)(mycrc & 0xFF)) && ((unsigned char)RxData[7]==(unsigned char)((mycrc>>8) & 0xFF))) { 
      return rxsize;
    } else return -1;
  }
  return 0;
}
//Converter array char in float
float ConvertFloat(char *buff) {
  unsigned char cValue[4];
  float *value;
  
  cValue[0] = buff[5];
  cValue[1] = buff[4];
  cValue[2] = buff[3];
  cValue[3] = buff[2];
  value = (float *)cValue;
  return (float)(*value);
}
//Converter array char in long
long ConvertLong(char *buff) {
  long value;
  
  value = (((((unsigned long)buff[2]) & 0xFF)<<24) + ((((unsigned long)buff[3]) & 0xFF)<<16) + ((((unsigned long)buff[4]) & 0xFF)<<8) + (((unsigned long)buff[5]) & 0xFF));
  return (value);
}
//Calcola MD5
void GetMD5(char *md5, char *buff){
  HASH_SUM Hash;
  BYTE resmd[16];
  
  MD5Initialize (&Hash);
  HashAddData (&Hash, (BYTE *)buff, strlen(buff)); 
  MD5Calculate(&Hash, resmd);
  sprintf(md5,"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", 
  resmd[0],resmd[1],resmd[2],resmd[3],resmd[4],resmd[5],resmd[6],resmd[7],
  resmd[8],resmd[9],resmd[10],resmd[11],resmd[12],resmd[13],resmd[14],resmd[15]);
}

int HttpRequest(char *webip, char *webport, char *weburl, char *webquery,int webtimeout){
  DWORD         T0;
  unsigned int  rxsize  =0;
  
  if(OpenSocket(webip, webport, webtimeout)==0) {
    CloseSocket();
    return -10;
  }  
  sprintf(PostReturn, "POST %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: Mozilla/4.0\r\nContent-Length: %d\r\nContent-Type: application/x-www-form-urlencoded\r\n\r\n", weburl, webip,(int)strlen(webquery));
  TCPWrite (TcpID, PostReturn, (int)strlen(PostReturn));
  TCPWrite (TcpID, webquery, (int)strlen(webquery));
  TCPWrite (TcpID, "\r\n", 2);
  T0=TickGet();
  do {
    rxsize = TCPRxLen(TcpID); 
    if(rxsize>0){
      if(rxsize>sizeof(PostReturn)) rxsize=sizeof(PostReturn); 
      T0=TickGet();
      TCPRead (TcpID, PostReturn, rxsize);
      PostReturn[rxsize]=0;
    }
    if((TickGet()-T0) > (webtimeout*TICK_SECOND)) {
      CloseSocket();
      return -30;
    }
  } while(TCPisConn(TcpID) || TCPRxLen(TcpID)>0); 
  CloseSocket();
  if(strlen(PostReturn)>0) return strlen(PostReturn);
  return -30;
}

int ParserXML(char *xml_buffer, char *start_tag, char *end_tag, int remove_tag, char *string_return){
   auto char *start_pointer;
   auto char *end_pointer;
   auto char buff[128];
   buff[0]=0;
   start_pointer = strstr(xml_buffer, start_tag);
   if(start_pointer){
      end_pointer = strstr(start_pointer, end_tag);
      if(end_pointer){
        if (remove_tag)
          end_pointer += strlen( end_tag );
        else
          start_pointer += strlen( start_tag );
        memcpy(buff, start_pointer, (end_pointer-start_pointer));
        buff[(end_pointer)-start_pointer] = 0;
        string_return[0]=0;
        sprintf(string_return,buff);
        return(1);
      }  else
        return(-2);
   }
   return(-1);
}

/*

MAIN PROGRAM

*/
void FlyportTask() {
  int           res;
  int           MainState;
  
  DWORD         QueryTimer;
  DWORD         ReleTimer;
  DWORD         NtpTimer;
  //BYTE          AlertFlag = 0;
  int           ErrorFlag = 0;
  BYTE          SyncNTP   = 0;
  char          PostQuery[128];
  int           NextQuery = 60;
  char          md5[33];
  BYTE          sdToInit  = 1;
  t_RTCC        MyTime;
  
  //Inizializzazione Variabili
  inverterdata.Indirizzo=0;
  inverterdata.EnergiaAttiva=0;
  inverterdata.ProduzioneTotale=0;
  inverterdata.ProduzioneGiornaliera=0;
  inverterdata.TensioneCampo=0;
  inverterdata.CorrenteCampo=0;
  inverterdata.TensioneRete=0;
  inverterdata.CorrenteRete=0;
  inverterdata.FrequenzaRete=0;
  inverterdata.Stato=0;
  inverterdata.Allarme=0;
  inverterdata.TempCPU=0;
  inverterdata.TempIGBT=0;
  inverterdata.ReleAutoconsumo = RELE_STATE;
  ALERT_LED_OFF;
  MainState =0;
  
  
  IOInit(sddetect, inup); // SD-CS line
  
  NETSetParam(MY_IP_ADDR,      LAN_IP);
  NETSetParam(MY_IP_ADDR,      LAN_IP);
  NETSetParam(SUBNET_MASK,     LAN_MASK);
  NETSetParam(PRIMARY_DNS,     LAN_DNS1);
  NETSetParam(SECONDARY_DNS,   LAN_DNS2);
  NETSetParam(MY_GATEWAY,      LAN_GW);
  NETSetParam(DHCP_ENABLE,     DISABLED);
  #ifdef OPENPICUS_WIFI
    NETSetParam(SSID_NAME,       WI_FI_SSID);
    NETSetParam(NETWORK_TYPE,    INFRASTRUCTURE);
    WFSetSecurity(WI_FI_SECURITY, WI_FI_PASS_PHRASE, strlen(WI_FI_PASS_PHRASE),0);
    WFConnect(WF_CUSTOM);
  #else
    ETHRestart(ETH_CUSTOM);
  #endif
  
  vTaskDelay(50);
  
  #ifdef DEBUG
    UARTWrite(PORT_RS232,"***********************************************\r\n");
    sprintf(debug, "%s %s\r\n",_APP_NAME, _APP_VER);
    UARTWrite(PORT_RS232,debug);
    UARTWrite(PORT_RS232,"***********************************************\r\n");
  #endif

  while(1){
    //if(!MAClinked);
    #ifdef OPENPICUS_WIFI
      if (WFStatus != CONNECTED){  //Attendo connessione Wi-Fi
      ERROR_LED_ON;
      vTaskDelay(100);
      #ifdef DEBUG
        UARTWrite(PORT_RS232,"ATTENZIONE - Connessione Wi-Fi in corso....\r\n");
      #endif
	  
	#else
      if(!MACLinked) {
      ERROR_LED_ON;
      vTaskDelay(100);
      #ifdef DEBUG
        UARTWrite(PORT_RS232,"ATTENZIONE - Connessione Ethernet non presente....\r\n");
      #endif
    #endif

      STATUS_LED_OFF;
    } else {  //Connessione Wi-Fi OK
      ERROR_LED_OFF;
      switch (MainState) {
        case 0:  //Sincronizzazione RTC con Server DayTime
          if(!InternetDayTime(NTP_SRV)) {
            SyncNTP   = 0;
            ErrorFlag |= (0x1<<0);
            #ifdef DEBUG
              UARTWrite(PORT_RS232, "ERRORE - Sincronizzazione NTP\r\n");
            #endif
          } else {
            SyncNTP = 1;
            ErrorFlag &= (~(0x1<<0));
            #ifdef DEBUG
              UARTWrite(PORT_RS232, "Sincronizzazione NTP OK\r\n");
            #endif
          }
          MainState = 100;
        break;
        
        case 1:  //Inizializzazione
          MainState ++;
        break;
        
        case 2: //Connessione con l'inverter
          #ifdef DEBUG
            sprintf(debug, "Connessione TCP/IP con inverter %s:%s\r\n",INVERTER_IP, INVERTER_PORT);
            UARTWrite(PORT_RS232,debug);
          #endif
          
          if(OpenSocket(INVERTER_IP,INVERTER_PORT, INVERTER_TIMEOUT)==0){
            #ifdef DEBUG
              UARTWrite(PORT_RS232,"ERRORE - Connessione TCP/IP con inverter fallita!\r\n");
            #endif
            MainState = 99;
            continue;
          }
          MainState++;
        break;
        
        case 3:
          if(inverterdata.Indirizzo==0){
            for (inverterdata.Indirizzo=INVERTER_ADDRESS; inverterdata.Indirizzo<=10; inverterdata.Indirizzo++){
              #ifdef DEBUG
                sprintf(debug, "Ricerca Inverter all'Indirizzo 0x%02X\r\n", inverterdata.Indirizzo);
                UARTWrite(PORT_RS232,debug);
              #endif
              ALERT_LED_ON;
              CMD_SN((unsigned char)inverterdata.Indirizzo)
              res=SendCmd(TxData,2);
              ALERT_LED_OFF;
              if(res==8){
                #ifdef DEBUG
                  sprintf(debug, "Inverter Trovato all'Indirizzo 0x%02X\r\n", inverterdata.Indirizzo);
                  UARTWrite(PORT_RS232,debug);
                #endif
                MainState ++;
                break;
              } else { //Nessuna Risposta
                vTaskDelay(100);
              }
            }
            if(MainState==3){
              #ifdef DEBUG
                UARTWrite(PORT_RS232,"ATTENZIONE - Nessun Inverter trovato\r\n");
              #endif
              inverterdata.Indirizzo=0;
              MainState=99;
            }
          } else MainState ++;
        break;
        
        case 4: //Recupero Energia Attiva o Grid Power
          CMD_EA((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.EnergiaAttiva =(int)ConvertFloat(RxData);
            MainState ++;
          } else  MainState=99;
        break;
        
        case 5: //Recupero Total Energy
          CMD_PT((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.ProduzioneTotale = ((float)ConvertLong(RxData))/1000;
            MainState++;
            CMD_PG((unsigned char)inverterdata.Indirizzo)
            STATUS_LED_ON;
            res=SendCmd(TxData,2);
            STATUS_LED_OFF;
            inverterdata.ProduzioneGiornaliera = ((float)ConvertLong(RxData))/1000;
          } else  MainState=99;
        break;
        
        case 6: //Recupero la tensione di campo 1 e 2
          CMD_TC1((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.TensioneCampo=ConvertFloat(RxData);
            CMD_TC2((unsigned char)inverterdata.Indirizzo)
            STATUS_LED_ON;
            res=SendCmd(TxData,2);
            STATUS_LED_OFF;
            if(res==8){
              inverterdata.TensioneCampo+=ConvertFloat(RxData);
              MainState++;
            } else  MainState=99;
          } else  MainState=99;
        break;
        
        case 7: //Recupero la corrente di campo 1 e 2
          CMD_CC1((unsigned char)inverterdata.Indirizzo)
          res=SendCmd(TxData,2);
          if(res==8){
            inverterdata.CorrenteCampo=ConvertFloat(RxData);
            CMD_CC2((unsigned char)inverterdata.Indirizzo)
            STATUS_LED_ON;
            res=SendCmd(TxData,2);
            STATUS_LED_OFF;
            if(res==8){
              inverterdata.CorrenteCampo+=ConvertFloat(RxData);
              MainState++;
            } else  MainState=99;
          } else  MainState=99;
        break;
        
        case 8: //Recupero la tensione di rete
          CMD_TR((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.TensioneRete=ConvertFloat(RxData);
            MainState++;
          } else  MainState=99;
        break;
        
        case 9: //Recupero la corrente di rete
          CMD_CR((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.CorrenteRete=ConvertFloat(RxData);
            MainState++;
          } else  MainState=99;
        break;
        
        case 10: //Recupero la Frequenza di rete
          CMD_FR((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.FrequenzaRete=ConvertFloat(RxData);
            MainState++;
          } else  MainState=99;
        break;
        
        case 11: //Recupero Stato e Allarme dell'inverter
          CMD_ST((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.Stato=(RxData[1]& 0x00FF);
            inverterdata.Allarme=(RxData[5] & 0x00FF);
            MainState++;
          } else  MainState=99;
        break;
        
        case 12: //Recupero Temperatura CPU
          CMD_TC((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.TempCPU=ConvertFloat(RxData);
            MainState++;
          } else  MainState=99;
        break;
        
        case 13: //Recupero Temperatura IGBT
          CMD_TI((unsigned char)inverterdata.Indirizzo)
          STATUS_LED_ON;
          res=SendCmd(TxData,2);
          STATUS_LED_OFF;
          if(res==8){
            inverterdata.TempIGBT=ConvertFloat(RxData);
            MainState++;
          } else  MainState=99;
        break;
        
        case 14: 
          CloseSocket();
          ErrorFlag &= (~(0x1<<1));
          if(inverterdata.EnergiaAttiva>0) 
            STATUS_LED_ON;
          else
            STATUS_LED_OFF;
            
          #ifdef DEBUG
            UARTWrite(PORT_RS232,"Dati dall'inverter:\r\n");
            sprintf(debug, "  Energia Attiva %d W\r\n", inverterdata.EnergiaAttiva);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Produzione Attiva %.2f Kwh\r\n", (double)inverterdata.ProduzioneTotale);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Tensione di Campo %.2f V\r\n", (double)inverterdata.TensioneCampo);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Corrente di Campo %.2f A\r\n", (double)inverterdata.CorrenteCampo);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Tensione di rete %.2f v\r\n", (double)inverterdata.TensioneRete);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Corrente di rete %.2f A\r\n", (double)inverterdata.CorrenteRete);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Frequenza di rete %.2f Hz\r\n", (double)inverterdata.FrequenzaRete);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Global State %d Allarme %d\r\n", inverterdata.Stato, inverterdata.Allarme );
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Temperatura CPU %.2f C\r\n", (double)inverterdata.TempCPU);
            UARTWrite(PORT_RS232,debug);
            sprintf(debug, "  Temperatura IGBT %.2f C\r\n", (double)inverterdata.TempIGBT);
            UARTWrite(PORT_RS232,debug);
          #endif
          RTCCRead(&MyTime);
          sprintf(inverterdata.UpdateDataTime, "%02d:%02d %02d/%02d/2%03d",  MyTime.hour, MyTime.min, MyTime.day, MyTime.month, MyTime.year);
          //Gestione RELE' Autoconsumo
          if(inverterdata.EnergiaAttiva>=SOGLIA_AUTOCONSUMO){
            RELE_ON;
            inverterdata.ReleAutoconsumo = 1;
            #ifdef DEBUG
              UARTWrite(PORT_RS232,"Rele' Autoconsumo ON\r\n");
            #endif
            ReleTimer=TickGet();
          }
          MainState++;
        break;
          
        case 15: //Memorizzazione SD
          #ifdef DEBUG
            UARTWrite(PORT_RS232,"Scrittura SD\r\n");
          #endif
          if (!IOGet(sddetect)){       //0 SD inserita 1 SD assente
            if (sdToInit){             //SD da inizializzare!
              if(sd_Init()>0) sdToInit = 0;   //SD inizializzata 
            }
          } else {
            sdToInit = 1;    //SD rimossa o assente!
            #ifdef DEBUG
              UARTWrite(PORT_RS232,"SD assente\r\n");
            #endif
          }
          if(sdToInit==0){ //Salvataggio in SD
            RTCCRead(&MyTime);
            sprintf(PostQuery, "%02d/%02d/2%03d;%02d:%02d:%02d;%d;%.2f;%.2f;%.2f;%.2f;%.2f;%.2f;%d;%d;%.2f;%.2f\r\n",
            MyTime.day, MyTime.month, MyTime.year, MyTime.hour, MyTime.min, MyTime.sec,
            inverterdata.EnergiaAttiva,(double)inverterdata.ProduzioneTotale,(double)inverterdata.TensioneCampo,
            (double)inverterdata.CorrenteCampo,(double)inverterdata.TensioneRete,  
            (double)inverterdata.CorrenteRete,(double)inverterdata.FrequenzaRete,
            inverterdata.Stato, inverterdata.Allarme,(double)inverterdata.TempCPU,(double)inverterdata.TempIGBT);
            sprintf(debug, "Inverter_%02d_%02d_2%03d.log", MyTime.day, MyTime.month, MyTime.year);
            write_File(debug, PostQuery, strlen(PostQuery), 0);
            #ifdef DEBUG
              UARTWrite(PORT_RS232,"Scrittura SD OK!\r\n");
            #endif
            
          }
          MainState++;
        break;
          
        case 16: //Trasmissione POST
          MainState=100;
          if((strlen(WEB_IP)<7) || strlen(WEB_PORT)==0) continue;
          #ifdef DEBUG
            sprintf(debug, "Trasmissione POST HTTP %s:%s\r\n", WEB_IP, WEB_PORT);
            UARTWrite(PORT_RS232,debug);
          #endif
          sprintf(PostQuery, "DATA=%d;%.2f;%.2f;%.2f;%.2f;%.2f;%.2f;%d;%d;%.2f;%.2f&PW=%s",
          inverterdata.EnergiaAttiva,(double)inverterdata.ProduzioneTotale,(double)inverterdata.TensioneCampo,
          (double)inverterdata.CorrenteCampo,(double)inverterdata.TensioneRete,
          (double)inverterdata.CorrenteRete,(double)inverterdata.FrequenzaRete,
          inverterdata.Stato, inverterdata.Allarme,(double)inverterdata.TempCPU,(double)inverterdata.TempIGBT,TXHTTP_PASSWORD);
          GetMD5(md5, PostQuery); //Calcolo MD5 della query 
          PostQuery[strlen(PostQuery)-strlen(TXHTTP_PASSWORD)]=0;
          strcat(PostQuery,md5);
          res=HttpRequest(WEB_IP, WEB_PORT, WEB_URL, PostQuery, 30);           //Trasmissione richiesta HTTP
          if(res<0){
            ErrorFlag |= (0x1<<2);
            #ifdef DEBUG
              sprintf(debug, "ERRORE - HTTP fallita CODE %d\r\n",res);
              UARTWrite(PORT_RS232,debug);
            #endif
            
            //ErrorFlag = 3;
          } else {
            if(ParserXML (PostReturn, "<NEXT>", "</NEXT>", 0, RxData)>0) {             //Analisi della risposta
              NextQuery = atoi(RxData);
              ErrorFlag &= (~(0x1<<2));
              #ifdef DEBUG
                UARTWrite(PORT_RS232,"Trasmissione HTTP completata con successo.\r\n");
                sprintf(debug, "Prossima trasmissione %d Sec\r\n", NextQuery);
                UARTWrite(PORT_RS232,debug);
              #endif
            } else {
              if(ParserXML (PostReturn, "<ERROR>", "</ERROR>", 0, RxData)>0) {
                ErrorFlag |= (0x1<<2);
                #ifdef DEBUG
                  sprintf(debug, "ERRORE Trasmissione HTTP fallita CODE %s\r\n", RxData);
                  UARTWrite(PORT_RS232,debug);
                #endif
                //ErrorFlag = 3;
              } else {
                ErrorFlag |= (0x1<<2);
                #ifdef DEBUG
                  sprintf(debug, "ERRORE Trasmissione HTTP fallita\r\n");
                  UARTWrite(PORT_RS232,debug);
                #endif
                //ErrorFlag = 3;
              }
            }
          }
        break;
        
        case 99: //Connessione con l'inverter fallita
          CloseSocket();
          ErrorFlag                    |= (0x1<<1);
          inverterdata.EnergiaAttiva    = 0;
          inverterdata.TensioneCampo    = 0;
          inverterdata.CorrenteCampo    = 0;
          inverterdata.TensioneRete     = 0;
          inverterdata.CorrenteRete     = 0;
          inverterdata.FrequenzaRete    = 0;
          inverterdata.Stato            = 0;
          inverterdata.Allarme          = 0;
          inverterdata.TempCPU          = 0;
          inverterdata.TempIGBT         = 0;
          inverterdata.ReleAutoconsumo  = RELE_STATE;
          NextQuery                     = 60;
          MainState                     = 100;
          STATUS_LED_OFF;
          #ifdef DEBUG
            UARTWrite(PORT_RS232,"Inverter non connesso o offline\r\n");
          #endif
        break;
        
        //Gestione Timer
        case 100:
          if((TickGet() - QueryTimer) > (NextQuery * TICK_SECOND)) {
            QueryTimer = TickGet();
            MainState = 2;
            continue;
          }
          if((TickGet() - ReleTimer) > (TIMEOUT_AUTOCONSUMO*TICK_SECOND)) {
            RELE_OFF;
            inverterdata.ReleAutoconsumo = 0;
          }
          if((TickGet() - NtpTimer) > (60 * TICK_SECOND)) {
            RTCCRead(&MyTime);
            if(MyTime.hour==4 && MyTime.min==0) MainState = 0;
            if(SyncNTP==0)                      MainState = 0;
            NtpTimer = TickGet();
          }
          if (IOGet(sddetect)) sdToInit = 1; //Controlla la rimozione della SD
          if(ErrorFlag >0)
            ERROR_LED_ON;
          else{
            if(inverterdata.EnergiaAttiva >0) STATUS_LED_ON;
          }
          vTaskDelay(20);
          ERROR_LED_OFF;
          if((ErrorFlag >0) && (inverterdata.EnergiaAttiva >0)) 
            STATUS_LED_ON;
          vTaskDelay(20);
        break;
      }
    }
  }
}

